package me.unurled.sacredrealms.sr.commands.admin;

import static me.unurled.sacredrealms.sr.utils.Component.NOT_PLAYER;
import static me.unurled.sacredrealms.sr.utils.Component.comp;
import static me.unurled.sacredrealms.sr.utils.Component.textComp;

import io.papermc.paper.command.brigadier.BasicCommand;
import io.papermc.paper.command.brigadier.CommandSourceStack;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import me.unurled.sacredrealms.sr.components.entity.EntityManager;
import me.unurled.sacredrealms.sr.components.entity.SREntityType;
import me.unurled.sacredrealms.sr.gui.entitytype.EntityTypeGUI;
import me.unurled.srcore.api.Manager;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import xyz.xenondevs.invui.window.Window;

public class EntityTypeCommand implements BasicCommand {

  private static void list(@NotNull CommandSender sender, @NotNull String @NotNull [] args) {
    if (!(sender instanceof Player p)) {
      sender.sendMessage(comp(NOT_PLAYER));
      return;
    }
    EntityManager em = Manager.getInstance(EntityManager.class);
    if (em.getTypes().isEmpty()) {
      sender.sendMessage(textComp("There are no entity types to display."));
      return;
    }

    Window window =
        Window.single().setViewer(p).setTitle("List").setGui(EntityTypeGUI.listGui()).build();
    window.open();
  }

  private static void edit(@NotNull CommandSender sender, @NotNull String @NotNull [] args) {
    if (args.length < 2) {
      sender.sendMessage("Usage: /entitytype edit <ID>");
      return;
    }
    if (!(sender instanceof Player p)) {
      sender.sendMessage(comp(NOT_PLAYER));
      return;
    }
    if (args.length < 3) {
      EntityManager em = Manager.getInstance(EntityManager.class);
      if (!em.getTypes().stream().map(SREntityType::getId).toList().contains(args[1])) {
        sender.sendMessage("Invalid entity type.");
        return;
      }
      SREntityType type = em.getType(args[1]);
      Window window =
          Window.single()
              .setViewer(p)
              .setTitle(args[1])
              .setGui(EntityTypeGUI.createGui(type))
              .build();
      window.open();
      return;
    }
    if (args.length < 4) {
      sender.sendMessage("Usage: /entitytype edit <ID> <type|behavior|loot|item|stats|name|level>");
      return;
    }
    if (args[2].equalsIgnoreCase("type")) {
      if (args.length < 5) {
        sender.sendMessage("Usage: /entitytype edit <ID> type <type>");
        return;
      }
      try {
        EntityType type = EntityType.valueOf(args[4]);
        EntityManager em = Manager.getInstance(EntityManager.class);
        if (setType(sender, args, em, type)) {}
      } catch (IllegalArgumentException e) {
        sender.sendMessage("Invalid entity type.");
      }
    }
  }

  private static void delete(@NotNull CommandSender sender, @NotNull String @NotNull [] args) {
    if (args.length < 2) {
      sender.sendMessage("Usage: /entitytype delete <ID>");
      return;
    }
    EntityManager em = Manager.getInstance(EntityManager.class);
    if (!em.getTypes().stream().map(SREntityType::getId).toList().contains(args[1])) {
      sender.sendMessage("Invalid entity type.");
    }
    //    em.removeEntityType(em.getType(args[1]));
  }

  private static void create(@NotNull CommandSender sender, @NotNull String @NotNull [] args) {
    if (args.length < 2) {
      sender.sendMessage("Usage: /entitytype create <Name>");
      return;
    }
    EntityManager em = Manager.getInstance(EntityManager.class);
    if (em.getTypes().stream().map(SREntityType::getId).toList().contains(args[1])) {
      sender.sendMessage("An entity type with that ID already exists.");
      return;
    }
    SREntityType type = new SREntityType(args[1], args[1].toUpperCase());
    em.addEntityType(type);
    sender.sendMessage(textComp("<green>Entity type created successfully."));
    if (sender instanceof Player p) {
      // opens gui
      Window window =
          Window.single()
              .setViewer(p)
              .setTitle(args[1])
              .setGui(EntityTypeGUI.createGui(type))
              .build();
      window.open();
    }
  }

  private static boolean setType(
      @NotNull CommandSender sender,
      @NotNull String @NotNull [] args,
      EntityManager em,
      EntityType type) {
    try {
      em.getType(args[1]).setType(type);
    } catch (Exception e) {
      sender.sendMessage("An error occurred while setting the entity type.");
      return true;
    }
    return false;
  }

  /**
   * Executes the command with the given {@link CommandSourceStack} and arguments.
   *
   * @param commandSourceStack the commandSourceStack of the command
   * @param args the arguments of the command ignoring repeated spaces
   */
  @Override
  public void execute(@NotNull CommandSourceStack commandSourceStack, @NotNull String[] args) {
    CommandSender sender = commandSourceStack.getSender();
    if (!sender.hasPermission("sr.entitytype")) {
      sender.sendMessage("You do not have permission to use this command.");
      return;
    }

    if (args.length == 0) {
      sender.sendMessage("Usage: /entitytype <create|delete|edit|list>");
      return;
    }

    switch (args[0].toLowerCase()) {
      case "create" -> create(sender, args);
      case "delete" -> delete(sender, args);
      case "edit" -> edit(sender, args);
      case "list" -> list(sender, args);
      default -> sender.sendMessage("Usage: /entitytype <create|delete|edit|list>");
    }
  }

  /**
   * Suggests possible completions for the given command {@link CommandSourceStack} and arguments.
   *
   * @param commandSourceStack the commandSourceStack of the command
   * @param args the arguments of the command including repeated spaces
   * @return a collection of suggestions
   */
  @Override
  public @NotNull Collection<String> suggest(
      @NotNull CommandSourceStack commandSourceStack, @NotNull String[] args) {
    CommandSender sender = commandSourceStack.getSender();
    if (sender.hasPermission("sr.entitytype")) {
      if (args.length == 1) {
        return List.of("create", "delete", "edit", "list");
      }
      if (args.length == 2) {
        return Manager.getInstance(EntityManager.class).getTypes().stream()
            .map(SREntityType::getId)
            .toList();
      }
      if (args.length == 3 && (args[0].equalsIgnoreCase("edit"))) {
        return List.of("type", "behavior", "loot", "item", "stats", "name", "level");
      }
      if (args.length == 4
          && (args[0].equalsIgnoreCase("edit") && (args[2].equalsIgnoreCase("type")))) {
        return Arrays.stream(EntityType.values())
            .filter(EntityType::isAlive)
            .map(EntityType::name)
            .toList();
      }
    }
    return List.of();
  }
}
