package me.unurled.sacredrealms.sr.commands.admin;

import io.papermc.paper.command.brigadier.BasicCommand;
import io.papermc.paper.command.brigadier.CommandSourceStack;
import java.util.Collection;
import java.util.List;
import me.unurled.sacredrealms.sr.components.player.PlayerManager;
import me.unurled.sacredrealms.sr.components.player.SRPlayer;
import me.unurled.srcore.api.Manager;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;

public class LevelCommand implements BasicCommand {

  /**
   * Executes the command with the given {@link CommandSourceStack} and arguments.
   *
   * @param commandSourceStack the commandSourceStack of the command
   * @param args the arguments of the command ignoring repeated spaces
   */
  @Override
  public void execute(@NotNull CommandSourceStack commandSourceStack, @NotNull String[] args) {
    CommandSender sender = commandSourceStack.getSender();
    if (!sender.hasPermission("sr.level")) {
      sender.sendMessage("You do not have permission to use this command.");
      return;
    }

    if (args.length != 2) {
      sender.sendMessage("Usage: /level <player> <level>");
      return;
    }

    Player target = sender.getServer().getPlayer(args[0]);

    if (target == null) {
      sender.sendMessage("Player not found.");
      return;
    }

    PlayerManager pm = Manager.getInstance(PlayerManager.class);
    SRPlayer player = pm.getPlayer(target.getUniqueId());

    if (player != null) {
      int level = player.getLevel();
      player.setLevel(Integer.parseInt(args[1]));
      // save player data
      pm.savePlayer(player);
      pm.levelUp(player, level);
    } else {
      sender.sendMessage("Player not found.");
    }
  }

  /**
   * Suggests possible completions for the given command {@link CommandSourceStack} and arguments.
   *
   * @param commandSourceStack the commandSourceStack of the command
   * @param args the arguments of the command including repeated spaces
   * @return a collection of suggestions
   */
  @Override
  public @NotNull Collection<String> suggest(
      @NotNull CommandSourceStack commandSourceStack, @NotNull String[] args) {
    return List.of();
  }
}
