package me.unurled.sacredrealms.sr.commands.admin;

import static me.unurled.sacredrealms.sr.utils.Component.NO_PERMISSION;
import static me.unurled.sacredrealms.sr.utils.Component.textComp;

import io.papermc.paper.command.brigadier.BasicCommand;
import io.papermc.paper.command.brigadier.CommandSourceStack;
import java.util.Collection;
import java.util.List;
import me.unurled.sacredrealms.sr.components.entity.EntityManager;
import me.unurled.sacredrealms.sr.components.entity.SREntityType;
import me.unurled.srcore.api.Manager;
import org.bukkit.World;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;

public class SpawnEntityCommand implements BasicCommand {

  private static final String USAGE =
      "Usage: /spawnentity <entityType> [amount] [x] [y] [z] " + "[world]";

  private void spawnEntity(
      CommandSender sender, String type, int amount, double x, double y, double z, World world) {
    EntityManager em = Manager.getInstance(EntityManager.class);
    if (em == null) {
      sender.sendMessage("EntityManager is null.");
      return;
    }
    SREntityType eType = em.getType(type);
    if (eType == null) {
      sender.sendMessage("Invalid entity type.");
      return;
    }
    em.spawnEntity(eType, amount, x, y, z, world);
    sender.sendMessage("Spawned " + amount + " " + eType.getName() + "s.");
  }

  private int getAmount(CommandSender sender, String @NotNull [] args) {
    if (args.length > 1 && (args.length < 5 || args.length > 6)) {
      try {
        return Integer.parseInt(args[1]);
      } catch (NumberFormatException e) {
        sender.sendMessage("Invalid amount.");
      }
    }
    return 1;
  }

  /**
   * Executes the command with the given {@link CommandSourceStack} and arguments.
   *
   * @param commandSourceStack the commandSourceStack of the command
   * @param args the arguments of the command ignoring repeated spaces
   */
  @Override
  public void execute(@NotNull CommandSourceStack commandSourceStack, @NotNull String[] args) {
    CommandSender sender = commandSourceStack.getSender();
    if (!sender.hasPermission("sr.spawn-entity")) {
      sender.sendMessage(textComp(NO_PERMISSION));
      return;
    }

    if (args.length == 0) {
      sender.sendMessage(USAGE);
      return;
    }
    double x;
    double y;
    double z;
    World world;
    if (args.length == 6) {
      if (sender instanceof Player p) {
        if (args[2].equals("~")) {
          x = p.getLocation().getX();
        } else {
          try {
            x = Double.parseDouble(args[2]);
          } catch (NumberFormatException e) {
            sender.sendMessage("Invalid x location.");
            return;
          }
        }
        if (args[3].equals("~")) {
          y = p.getLocation().getY();
        } else {
          try {
            y = Double.parseDouble(args[3]);
          } catch (NumberFormatException e) {
            sender.sendMessage("Invalid y location.");
            return;
          }
        }
        if (args[4].equals("~")) {
          z = p.getLocation().getZ();
        } else {
          try {
            z = Double.parseDouble(args[4]);
          } catch (NumberFormatException e) {
            sender.sendMessage("Invalid z location.");
            return;
          }
        }
        world = p.getWorld();
      } else {
        try {
          x = Double.parseDouble(args[2]);
          y = Double.parseDouble(args[3]);
          z = Double.parseDouble(args[4]);
          world = sender.getServer().getWorld(args[5]);
        } catch (NumberFormatException e) {
          sender.sendMessage("Invalid location.");
          return;
        }
      }
    } else {
      if (!(sender instanceof Player player)) {
        sender.sendMessage("You must set the location if you are not a player.");
        return;
      }
      x = player.getLocation().getX();
      y = player.getLocation().getY();
      z = player.getLocation().getZ();
      world = player.getWorld();
    }

    int amount = getAmount(sender, args);

    if (world == null) {
      sender.sendMessage("Invalid world.");
      return;
    }

    // spawn entity
    spawnEntity(sender, args[0], amount, x, y, z, world);
  }

  /**
   * Suggests possible completions for the given command {@link CommandSourceStack} and arguments.
   *
   * @param commandSourceStack the commandSourceStack of the command
   * @param args the arguments of the command including repeated spaces
   * @return a collection of suggestions
   */
  @Override
  public @NotNull Collection<String> suggest(
      @NotNull CommandSourceStack commandSourceStack, @NotNull String[] args) {
    CommandSender sender = commandSourceStack.getSender();
    if (sender.hasPermission("sr.spawn-entity")) {
      if (args.length == 1) {
        return Manager.getInstance(EntityManager.class).getTypes().stream()
            .map(SREntityType::getId)
            .toList();
      } else if (args.length == 2) {
        return List.of("1", "2", "3", "4", "5");
      } else if (args.length == 3) {
        return List.of("~");
      } else if (args.length == 4) {
        return List.of("~");
      } else if (args.length == 5) {
        return List.of("~");
      } else if (args.length == 6) {
        return sender.getServer().getWorlds().stream().map(World::getName).toList();
      }
    }
    return List.of();
  }
}
