package me.unurled.sacredrealms.sr.commands.player;

import static me.unurled.sacredrealms.sr.utils.Component.comp;

import io.papermc.paper.command.brigadier.BasicCommand;
import io.papermc.paper.command.brigadier.CommandSourceStack;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Objects;
import me.unurled.sacredrealms.sr.SR;
import me.unurled.sacredrealms.sr.components.player.PlayerManager;
import me.unurled.sacredrealms.sr.data.DataManager;
import me.unurled.srcore.api.Manager;
import org.bukkit.Bukkit;
import org.bukkit.OfflinePlayer;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;

public class ResetAdventureCommand implements BasicCommand {

  private static final Long COOLDOWN = 120L;

  private final HashMap<Player, Long> cooldowns = new HashMap<>();

  private void reset(@NotNull CommandSender sender, @NotNull String @NotNull [] args) {
    if (args.length > 2) {
      sender.sendMessage(comp("<red>Usage: /resetadventure <player>"));
      return;
    }
    if (sender instanceof Player p && (args.length == 0)) {
      boolean status = resetAdventure(p);
      if (status) {
        sender.sendMessage(comp("<green>Your adventure has been reset."));
      } else {
        sender.sendMessage(comp("<red>Failed to reset your adventure."));
      }
      return;
    }
    OfflinePlayer player = SR.getPlugin().getServer().getOfflinePlayer(args[0]);

    boolean status = resetAdventure(player);
    if (status) {
      sender.sendMessage(comp("<green>Successfully reset " + player.getName() + "'s adventure."));
    } else {
      sender.sendMessage(comp("<red>Failed to reset " + player.getName() + "'s adventure."));
    }
  }

  private boolean resetAdventure(OfflinePlayer player) {
    DataManager dataManager = Manager.getInstance(DataManager.class);
    if (dataManager != null) {
      if (dataManager.getDataHandler().exists(PlayerManager.PLAYER_KEY + player.getUniqueId())) {
        dataManager.getDataHandler().remove(PlayerManager.PLAYER_KEY + player.getUniqueId());
        return true;
      } else {
        return false;
      }
    }
    return false;
  }

  /**
   * Executes the command with the given {@link CommandSourceStack} and arguments.
   *
   * @param commandSourceStack the commandSourceStack of the command
   * @param args the arguments of the command ignoring repeated spaces
   */
  @Override
  public void execute(@NotNull CommandSourceStack commandSourceStack, @NotNull String[] args) {
    CommandSender sender = commandSourceStack.getSender();
    if (sender.hasPermission("sr.resetadventure")) {
      reset(sender, args);
      return;
    }
    if (args.length > 0) {
      sender.sendMessage(comp("<red>Usage: /resetadventure"));
      return;
    }
    if (sender instanceof Player player) {
      if (cooldowns.containsKey(player)) {
        long lastUsed = cooldowns.get(player);
        long secondsLeft = ((lastUsed / 1000) + COOLDOWN) - (System.currentTimeMillis() / 1000);
        if (secondsLeft > 0) {
          sender.sendMessage(
              comp(
                  "<red>You must wait "
                      + secondsLeft
                      + " seconds before using this command again."));
          return;
        }
      }
      boolean status = resetAdventure(player);
      if (status) {
        sender.sendMessage(comp("<green>Your adventure has been reset."));
      } else {
        sender.sendMessage(comp("<red>Failed to reset your adventure."));
      }
      cooldowns.put(player, System.currentTimeMillis());
    }
  }

  /**
   * Suggests possible completions for the given command {@link CommandSourceStack} and arguments.
   *
   * @param commandSourceStack the commandSourceStack of the command
   * @param args the arguments of the command including repeated spaces
   * @return a collection of suggestions
   */
  @Override
  public @NotNull Collection<String> suggest(
      @NotNull CommandSourceStack commandSourceStack, @NotNull String[] args) {
    CommandSender sender = commandSourceStack.getSender();
    if (sender.hasPermission("sr.resetadventure")) {
      if (args.length == 1) {
        return null;
      }
      if (args[0].isBlank()) {
        return Arrays.stream(Bukkit.getOfflinePlayers())
            .map(OfflinePlayer::getName)
            .toList()
            .stream()
            .limit(10)
            .toList();
      }
      return Arrays.stream(Bukkit.getOfflinePlayers())
          .map(OfflinePlayer::getName)
          .filter(Objects::nonNull)
          .filter(name -> name.toLowerCase().startsWith(args[0].toLowerCase()))
          .toList()
          .stream()
          .limit(10)
          .toList();
    }
    return List.of();
  }
}
