package me.unurled.sacredrealms.sr.components.block;

import static me.unurled.sacredrealms.sr.utils.Component.textComp;
import static me.unurled.sacredrealms.sr.utils.Logger.error;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import me.unurled.sacredrealms.sr.constants.Keys;
import net.kyori.adventure.text.Component;
import org.bukkit.Material;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.jetbrains.annotations.NotNull;

public class BlockItem {

  private static final Material BLOCK_MATERIAL = Material.STONE;

  private final Component displayName;
  private final List<Component> lore;
  private final int customModelData;

  public BlockItem(@NotNull ItemStack itemStack) {
    this.displayName = itemStack.getItemMeta().displayName();
    this.lore = itemStack.getItemMeta().lore();
    this.customModelData = itemStack.getItemMeta().getCustomModelData();
  }

  public BlockItem(@NotNull Map<?, ?> configObject) {
    String rawDisplayName = (String) configObject.get(Keys.BLOCK_DISPLAYNAME);
    List<String> rawLore = new ArrayList<>();
    if (configObject.get(Keys.BLOCK_LORE) instanceof List) {
      try {
        rawLore = (List<String>) configObject.get(Keys.BLOCK_LORE);
      } catch (ClassCastException e) {
        error("Failed to cast lore to List<String> " + e.getMessage());
      }
    }

    this.displayName = textComp(rawDisplayName);
    this.customModelData = (int) configObject.get(Keys.BLOCK_CUSTOMMODELDATA);
    this.lore = new ArrayList<>();

    for (String loreItem : rawLore) {
      this.lore.add(textComp(loreItem));
    }
  }

  public BlockItem(Component displayName, List<Component> lore, int customModelData) {
    this.displayName = displayName;
    this.lore = lore;
    this.customModelData = customModelData;
  }

  public Component getDisplayName() {
    return this.displayName;
  }

  public ItemStack getItem(int amount) {
    ItemStack item = new ItemStack(BLOCK_MATERIAL, amount);
    ItemMeta itemMeta = item.getItemMeta();

    itemMeta.displayName(this.displayName);
    itemMeta.lore(this.lore);
    itemMeta.setCustomModelData(this.customModelData);

    item.setItemMeta(itemMeta);

    return item;
  }
}
