package me.unurled.sacredrealms.sr.components.block;

import static me.unurled.sacredrealms.sr.utils.Items.locationToString;

import java.util.HashMap;
import java.util.Map;
import me.unurled.sacredrealms.sr.SR;
import me.unurled.srcore.api.Manager;
import org.bukkit.Bukkit;
import org.bukkit.GameMode;
import org.bukkit.Location;
import org.bukkit.Sound;
import org.bukkit.World;
import org.bukkit.entity.ItemDisplay;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.NotNull;

public class BlockManager extends Manager {

  private final Map<String, Block> blocks = new HashMap<>();

  public void addBlock(@NotNull Location location, @NotNull BlockItem blockItem) {
    Block block = new Block(location, blockItem);
    blocks.put(locationToString(location), block);
  }

  public void removeBlock(@NotNull Location location) {
    String blockID = locationToString(location);

    if (blocks.containsKey(blockID)) {
      Block block = blocks.get(blockID);
      block.breakBlock();
      blocks.remove(blockID);
    }
  }

  public void breakBlock(Location loc, Player player) {
    String blockID = locationToString(loc);
    if (blocks.containsKey(blockID)) {
      Block block = blocks.get(blockID);
      block.breakBlock();

      player.playSound(loc, Sound.BLOCK_STONE_BREAK, 1, 1);

      if (player.getGameMode() == GameMode.SURVIVAL) {
        World world = loc.getWorld();
        world.dropItemNaturally(loc, block.getBlockItem().getItem(1));
      }

      blocks.remove(blockID);
    }
  }

  private @NotNull Location getLocationFromEntity(@NotNull Location location) {
    Location entityLocation = location.clone();
    entityLocation.setX(location.getX() - 0.5);
    entityLocation.setY(location.getY() - 0.5);
    entityLocation.setZ(location.getZ() - 0.5);

    return entityLocation;
  }

  @Override
  public void load() {
    Bukkit.getScheduler()
        .runTask(
            SR.getPlugin(),
            () -> {
              for (World world : SR.getPlugin().getServer().getWorlds()) {
                if (world == null) {
                  continue;
                }
                for (ItemDisplay entity : world.getEntitiesByClass(ItemDisplay.class)) {
                  Location location = getLocationFromEntity(entity.getLocation());
                  ItemStack itemStack = entity.getItemStack();
                  BlockItem blockItem = new BlockItem(itemStack);
                  entity.remove();
                  this.addBlock(location, blockItem);
                }
              }
            });
  }
}
