package me.unurled.sacredrealms.sr.components.clientbuild;

import com.google.gson.JsonElement;
import com.google.gson.JsonPrimitive;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;
import java.lang.reflect.Type;
import java.util.Map.Entry;
import me.unurled.sacredrealms.sr.utils.gson.LocationSerializer;
import org.bukkit.Location;
import org.bukkit.block.data.BlockData;
import org.jetbrains.annotations.NotNull;

public class ClientBuildSerializer implements JsonSerializer<ClientBuild> {

  /**
   * Gson invokes this call-back method during serialization when it encounters a field of the
   * specified type.
   *
   * <p>In the implementation of this call-back method, you should consider invoking {@link
   * JsonSerializationContext#serialize(Object, Type)} method to create JsonElements for any
   * non-trivial field of the {@code src} object. However, you should never invoke it on the {@code
   * src} object itself since that will cause an infinite loop (Gson will call your call-back method
   * again).
   *
   * @param src the object that needs to be converted to Json.
   * @param typeOfSrc the actual type (fully genericized version) of the source object.
   * @param context
   * @return a JsonElement corresponding to the specified object.
   */
  @Override
  public JsonElement serialize(
      @NotNull ClientBuild src, Type typeOfSrc, JsonSerializationContext context) {
    StringBuilder s = new StringBuilder();
    s.append(src.getName());
    s.append(";");
    LocationSerializer locSer = new LocationSerializer();
    for (Entry<Location, BlockData> entry : src.getBlocks().entrySet()) {
      s.append("{");
      JsonElement j = locSer.serialize(entry.getKey(), Location.class, context);
      s.append(j);
      s.append("|");
      s.append(entry.getValue().getAsString());
      s.append("},");
    }
    return new JsonPrimitive(s.toString());
  }
}
