package me.unurled.sacredrealms.sr.components.cutscene;

import static me.unurled.sacredrealms.sr.utils.Component.textComp;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.UUID;
import me.unurled.srcore.api.Manager;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class CutsceneManager extends Manager {
  // TODO: spectator to hide toolbar (make it ride an invisible entity to prevent movement)
  // TODO: save modifications if any
  // TODO: do you set player back to original position ?
  // TODO: make a system to easily create cutscenesw

  private List<Cutscene> cutscenes;

  // cutscene id, player uuid
  private Map<String, UUID> currentPlayingCutscenes;

  private Map<UUID, PlayerOrigin> playerOrigins;

  private List<String> cutsceneIds;

  /** Load the manager */
  @Override
  public void load() {
    super.load();
    cutscenes = new ArrayList<>();
    currentPlayingCutscenes = new HashMap<>();
    playerOrigins = new HashMap<>();
    cutsceneIds = new ArrayList<>();
  }

  /** Unload the manager */
  @Override
  public void unload() {
    super.unload();
    cutscenes.clear();
    currentPlayingCutscenes.clear();
    playerOrigins.clear();
    cutsceneIds.clear();
  }

  public void createCutscene(String id) {
    Cutscene cutscene = new Cutscene(id, id);
    addCutscene(cutscene);
  }

  public void removeCutscene(String id) {
    Cutscene cutscene = getCutscene(id);
    removeCutscene(cutscene);
  }

  public void addCutscene(Cutscene cutscene) {
    cutscenes.add(cutscene);
    cutsceneIds.add(cutscene.getId());
  }

  public void removeCutscene(Cutscene cutscene) {
    cutscenes.remove(cutscene);
    cutsceneIds.remove(cutscene.getId().toUpperCase());
  }

  public void addCurrentPlayingCutscene(@NotNull Cutscene cutscene, @NotNull Player p) {
    currentPlayingCutscenes.put(cutscene.getId(), p.getUniqueId());
  }

  public void removeCurrentPlayingCutscene(@NotNull Cutscene cutscene, @NotNull Player p) {
    currentPlayingCutscenes.remove(cutscene.getId(), p.getUniqueId());
  }

  public List<String> getCutscenesId() {
    return cutsceneIds;
  }

  public void stopCutscene(Player p) {
    for (String string : currentPlayingCutscenes.keySet()) {
      if (currentPlayingCutscenes.get(string).equals(p.getUniqueId())) {
        currentPlayingCutscenes.remove(string, p.getUniqueId());
        Cutscene cutscene = getCutscene(string);
        if (cutscene != null) {
          cutscene.stop(p);
          PlayerOrigin.restore(playerOrigins.get(p.getUniqueId()), p);
          playerOrigins.remove(p.getUniqueId());
        }
        return;
      }
    }
  }

  public void stopCutscenesForAllPlayers() {
    for (String string : currentPlayingCutscenes.keySet()) {
      Cutscene cutscene = getCutscene(string);
      if (cutscene != null) {
        UUID uuid = currentPlayingCutscenes.get(string);
        cutscene.stop(uuid);
        currentPlayingCutscenes.remove(cutscene.getId(), uuid);
        Player player = Bukkit.getPlayer(uuid);
        if (player == null) return;
        PlayerOrigin.restore(playerOrigins.get(uuid), player);
        playerOrigins.remove(uuid);
      }
    }
  }

  public void playCutscene(Cutscene cutscene, Player p) {
    if (cutscene == null) {
      p.sendMessage(textComp("<red>There was an error with the cutscene!"));
      return;
    }
    if (cutscenes.contains(cutscene) && !currentPlayingCutscenes.containsValue(p.getUniqueId())) {
      currentPlayingCutscenes.put(cutscene.getId(), p.getUniqueId());
      setPlayerOrigins(p);
      cutscene.play(p);
    }
  }

  @Nullable
  public Cutscene getCutscene(Player p) {
    for (Entry<String, UUID> entry : currentPlayingCutscenes.entrySet()) {
      if (entry.getValue().equals(p.getUniqueId())) return getCutscene(entry.getKey());
    }
    return null;
  }

  @Nullable
  public Cutscene getCutscene(@NotNull String id) {
    if (!cutsceneIds.contains(id.toUpperCase())) {
      return null;
    }
    for (Cutscene cutscene : cutscenes) {
      if (cutscene.getId().equalsIgnoreCase(id.toUpperCase())) return cutscene;
    }
    return null;
  }

  public boolean isPlayingCutscene(@NotNull Player p) {
    return currentPlayingCutscenes.containsValue(p.getUniqueId());
  }

  @Nullable
  public String getPlayingCutscene(@NotNull Player p) {
    for (Entry<String, UUID> entry : currentPlayingCutscenes.entrySet()) {
      if (entry.getValue().equals(p.getUniqueId())) return entry.getKey();
    }
    return null;
  }

  public void setPlayerOrigins(@NotNull Player p) {
    playerOrigins.put(
        p.getUniqueId(),
        new PlayerOrigin(
            p.getLocation(),
            p.getAllowFlight(),
            p.isFlying(),
            p.getWalkSpeed(),
            p.getFlySpeed(),
            p.getGameMode()));
  }
}
