package me.unurled.sacredrealms.sr.components.cutscene;

import static me.unurled.sacredrealms.sr.utils.Logger.error;

import com.google.gson.JsonElement;
import com.google.gson.JsonPrimitive;
import java.util.Map;
import me.unurled.sacredrealms.sr.utils.gson.LocationSerializer;
import org.bukkit.Location;
import org.bukkit.block.data.BlockData;
import org.jetbrains.annotations.Contract;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * Frame class
 *
 * <p>example of serializable frame:
 *
 * <pre>{@code
 * {
 *   "id": 1,
 *   "entities": "entities serializable",
 *   "blocks": "blocks serializable",
 *   "action": "action", // if type is entity
 *   "location": {
 *      "world": "world",
 *      "x": 0,
 *      "y": 0,
 *      "z": 0,
 *      "yaw": 0,
 *      "pitch": 0
 *   }
 * }
 * }</pre>
 */
public class Frame {
  private static int idCounter = 0;
  private final Location location;
  private final int id;
  @Nullable private Map<Location, BlockData> blocks;

  public Frame(Location location) {
    this.location = location;
    id = idCounter;
    idCounter++;
    blocks = null;
  }

  public Frame(Location location, int ID) {
    this.location = location;
    this.id = ID;
    blocks = null;
  }

  @Contract("_ -> new")
  public static @NotNull Frame deserialize(@NotNull JsonElement jsonElement) {
    int id = jsonElement.getAsJsonObject().get("id").getAsInt();
    Location loc = null;
    try {
      LocationSerializer locationDeserializer = new LocationSerializer();
      loc =
          locationDeserializer.deserialize(
              jsonElement.getAsJsonObject().get("location"), Location.class, null);
    } catch (Exception e) {
      error("Error deserializing location in Frame with ID: " + id);
    }

    Frame frame = new Frame(loc, id);

    try {
      frame.setBlocks((Map<Location, BlockData>) jsonElement.getAsJsonObject().get("blocks"));
    } catch (Exception e) {
      error("Error deserializing blocks in Frame with ID: " + id);
    }
    return frame;
  }

  public Location getLocation() {
    return location;
  }

  /** Play the frame aka move stuff around :shrug: */
  public void play() {}

  public int getId() {
    return id;
  }

  public @Nullable Map<Location, BlockData> getBlocks() {
    return blocks;
  }

  public void setBlocks(@Nullable Map<Location, BlockData> blocks) {
    this.blocks = blocks;
  }

  public JsonElement serialize() {
    String s =
        "{"
            + "\"id\":"
            + id
            + ","
            + ","
            + "\"blocks\":"
            + blocks
            + ","
            + "\"location\":"
            + location.serialize()
            + "}";
    return new JsonPrimitive(s);
  }
}
