package me.unurled.sacredrealms.sr.components.cutscene;

import static me.unurled.sacredrealms.sr.utils.CutsceneUtil.interpolateFrames;

import java.util.ArrayList;
import java.util.List;
import org.bukkit.Location;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * Marker class
 *
 * <pre>{@code
 * {
 *   "id": 1,
 *   "frames": [ 1, 2, 3 ], // Frame IDs
 *   "timeBetweenFrames": 20,
 *   "overAllTime": 60,
 *   "start": {
 *      "world": "world",
 *      "x": 0,
 *      "y": 0,
 *      "z": 0,
 *      "yaw": 0,
 *      "pitch": 0
 *   }
 * }
 * }</pre>
 */
public class Marker {

  private static int idCounter = 0;
  private static List<Marker> markers = new ArrayList<>();
  private final int id;
  private List<Frame> frames;
  private Long timeBetweenFrames;
  private Long overAllTime;
  private Location start;

  public Marker(Integer id) {
    frames = new ArrayList<>();
    timeBetweenFrames = 0L;
    overAllTime = 0L;
    if (id == null) {
      this.id = idCounter;
      idCounter++;
    } else {
      this.id = id;
    }
  }

  public Marker(@NotNull List<Frame> frames, long timeBetweenFrames, Location start) {
    this.frames = frames;
    this.timeBetweenFrames = timeBetweenFrames;
    this.overAllTime = timeBetweenFrames * frames.size();
    this.start = start;
    id = idCounter;
    idCounter++;
  }

  public static List<Marker> getMarkers() {
    return markers;
  }

  public static void setMarkers(List<Marker> markers) {
    Marker.markers = markers;
  }

  public static @Nullable Marker getMarker(int ID) {
    for (Marker marker : markers) {
      if (marker.getId() == ID) {
        return marker;
      }
    }
    return null;
  }

  public List<Frame> getFrames() {
    return frames;
  }

  public void setFrames(List<Frame> frames) {
    this.frames = frames;
  }

  public Long getTimeBetweenFrames() {
    return timeBetweenFrames;
  }

  public void setTimeBetweenFrames(Long timeBetweenFrames) {
    this.timeBetweenFrames = timeBetweenFrames;
  }

  public Long getOverAllTime() {
    return overAllTime;
  }

  public void setOverAllTime(Long overAllTime) {
    this.overAllTime = overAllTime;
  }

  public Location getStart() {
    return start;
  }

  public void setStart(Location start) {
    this.start = start;
  }

  public void calculateFrames(Marker endMarker) {
    frames = interpolateFrames(this, endMarker, timeBetweenFrames);
  }

  public int getId() {
    return id;
  }

  public void addFrame(Frame frame) {
    frames.add(frame);
  }

  public void removeFrame(Frame frame) {
    frames.remove(frame);
  }

  public void removeFrame(int index) {
    frames.remove(index);
  }

  @Override
  public String toString() {
    return "Marker{"
        + "ID="
        + id
        + ", frames="
        + frames
        + ", timeBetweenFrames="
        + timeBetweenFrames
        + ", overAllTime="
        + overAllTime
        + ", start="
        + start
        + '}';
  }
}
