package me.unurled.sacredrealms.sr.components.pack.background;

import me.unurled.sacredrealms.sr.utils.character.CharRepo;
import me.unurled.sacredrealms.sr.utils.character.ConfiguredChar;
import org.jetbrains.annotations.Contract;
import org.jetbrains.annotations.NotNull;

public class Background {
  private ConfiguredChar left,
      offset1,
      offset2,
      offset4,
      offset8,
      offset16,
      offset32,
      offset64,
      offset128,
      right;
  private int leftMargin, rightMargin;

  private Background() {}

  public Background(
      ConfiguredChar left,
      ConfiguredChar offset1,
      ConfiguredChar offset2,
      ConfiguredChar offset4,
      ConfiguredChar offset8,
      ConfiguredChar offset16,
      ConfiguredChar offset32,
      ConfiguredChar offset64,
      ConfiguredChar offset128,
      ConfiguredChar right,
      int leftMargin,
      int rightMargin,
      boolean removeShadow) {
    this.left = left;
    this.offset1 = offset1;
    this.offset2 = offset2;
    this.offset4 = offset4;
    this.offset8 = offset8;
    this.offset16 = offset16;
    this.offset32 = offset32;
    this.offset64 = offset64;
    this.offset128 = offset128;
    this.right = right;
    this.leftMargin = leftMargin;
    this.rightMargin = rightMargin;
  }

  @Contract(value = " -> new", pure = true)
  public static @NotNull Builder builder() {
    return new Builder();
  }

  public ConfiguredChar getLeft() {
    return left;
  }

  public ConfiguredChar getOffset1() {
    return offset1;
  }

  public ConfiguredChar getOffset2() {
    return offset2;
  }

  public ConfiguredChar getOffset4() {
    return offset4;
  }

  public ConfiguredChar getOffset8() {
    return offset8;
  }

  public ConfiguredChar getOffset16() {
    return offset16;
  }

  public ConfiguredChar getOffset32() {
    return offset32;
  }

  public ConfiguredChar getOffset64() {
    return offset64;
  }

  public ConfiguredChar getOffset128() {
    return offset128;
  }

  public ConfiguredChar getRight() {
    return right;
  }

  public String getBackGroundImage(int n) {
    String offset = CharRepo.getShortestNegChars(n + rightMargin + 2);
    n = n + leftMargin + rightMargin + 2;
    StringBuilder stringBuilder = new StringBuilder();
    stringBuilder.append(left.getCharacter());
    while (n >= 128) {
      stringBuilder.append(CharRepo.NEG1.getCh());
      stringBuilder.append(offset128.getCharacter());
      n -= 128;
    }
    if (n - 64 >= 0) {
      stringBuilder.append(CharRepo.NEG1.getCh());
      stringBuilder.append(offset64.getCharacter());
      n -= 64;
    }
    if (n - 32 >= 0) {
      stringBuilder.append(CharRepo.NEG1.getCh());
      stringBuilder.append(offset32.getCharacter());
      n -= 32;
    }
    if (n - 16 >= 0) {
      stringBuilder.append(CharRepo.NEG1.getCh());
      stringBuilder.append(offset16.getCharacter());
      n -= 16;
    }
    if (n - 8 >= 0) {
      stringBuilder.append(CharRepo.NEG1.getCh());
      stringBuilder.append(offset8.getCharacter());
      n -= 8;
    }
    if (n - 4 >= 0) {
      stringBuilder.append(CharRepo.NEG1.getCh());
      stringBuilder.append(offset4.getCharacter());
      n -= 4;
    }
    if (n - 2 >= 0) {
      stringBuilder.append(CharRepo.NEG1.getCh());
      stringBuilder.append(offset2.getCharacter());
      n -= 2;
    }
    if (n - 1 >= 0) {
      stringBuilder.append(CharRepo.NEG1.getCh());
      stringBuilder.append(offset1.getCharacter());
    }
    stringBuilder.append(CharRepo.NEG1.getCh());
    stringBuilder.append(right.getCharacter());
    stringBuilder.append(offset);
    return stringBuilder.toString();
  }

  public static class Builder {

    private final Background background;

    public Builder() {
      this.background = new Background();
    }

    @Contract(value = " -> new", pure = true)
    public static @NotNull Builder of() {
      return new Builder();
    }

    public Builder left(ConfiguredChar configuredChar) {
      background.left = configuredChar;
      return this;
    }

    public Builder right(ConfiguredChar configuredChar) {
      background.right = configuredChar;
      return this;
    }

    public Builder offset1(ConfiguredChar configuredChar) {
      background.offset1 = configuredChar;
      return this;
    }

    public Builder offset2(ConfiguredChar configuredChar) {
      background.offset2 = configuredChar;
      return this;
    }

    public Builder offset4(ConfiguredChar configuredChar) {
      background.offset4 = configuredChar;
      return this;
    }

    public Builder offset8(ConfiguredChar configuredChar) {
      background.offset8 = configuredChar;
      return this;
    }

    public Builder offset16(ConfiguredChar configuredChar) {
      background.offset16 = configuredChar;
      return this;
    }

    public Builder offset32(ConfiguredChar configuredChar) {
      background.offset32 = configuredChar;
      return this;
    }

    public Builder offset64(ConfiguredChar configuredChar) {
      background.offset64 = configuredChar;
      return this;
    }

    public Builder offset128(ConfiguredChar configuredChar) {
      background.offset128 = configuredChar;
      return this;
    }

    public Builder leftMargin(int margin) {
      background.leftMargin = margin;
      return this;
    }

    public Builder rightMargin(int margin) {
      background.rightMargin = margin;
      return this;
    }

    public Background build() {
      return background;
    }
  }
}
