package me.unurled.sacredrealms.sr.components.pack.background;

import static me.unurled.sacredrealms.sr.utils.Logger.warn;

import java.io.File;
import java.util.HashMap;
import java.util.Map;
import me.unurled.sacredrealms.sr.SR;
import me.unurled.sacredrealms.sr.utils.character.CharacterArranger;
import me.unurled.sacredrealms.sr.utils.character.ConfiguredChar;
import me.unurled.srcore.api.Manager;
import org.jetbrains.annotations.NotNull;

public class BackgroundManager extends Manager {

  private final Map<String, Background> backgroundMap;

  public BackgroundManager() {
    this.backgroundMap = new HashMap<>();
  }

  @Override
  public void loadData() {
    File bgFolder = new File(SR.getPlugin().getDataFolder(), "backgrounds");
    if (!bgFolder.exists() && bgFolder.mkdirs()) {
      saveDefaultBackgrounds();
    }

    int height = 14;
    int ascent = 8;

    String key = "b0";

    Background background =
        Background.builder()
            .leftMargin(1)
            .rightMargin(1)
            .left(
                ConfiguredChar.builder()
                    .character(CharacterArranger.getAndIncrease())
                    .png("b0")
                    .height(14)
                    .ascent(7)
                    .width(1)
                    .build())
            .offset1(
                ConfiguredChar.builder()
                    .character(CharacterArranger.getAndIncrease())
                    .png("b1")
                    .height(height)
                    .ascent(ascent)
                    .width(1)
                    .build())
            .offset2(
                ConfiguredChar.builder()
                    .character(CharacterArranger.getAndIncrease())
                    .png("b2")
                    .height(height)
                    .ascent(ascent)
                    .width(1)
                    .build())
            .offset4(
                ConfiguredChar.builder()
                    .character(CharacterArranger.getAndIncrease())
                    .png("b4")
                    .height(height)
                    .ascent(ascent)
                    .width(1)
                    .build())
            .offset8(
                ConfiguredChar.builder()
                    .character(CharacterArranger.getAndIncrease())
                    .png("b8")
                    .height(height)
                    .ascent(ascent)
                    .width(1)
                    .build())
            .offset16(
                ConfiguredChar.builder()
                    .character(CharacterArranger.getAndIncrease())
                    .png("b16")
                    .height(height)
                    .ascent(ascent)
                    .width(1)
                    .build())
            .offset32(
                ConfiguredChar.builder()
                    .character(CharacterArranger.getAndIncrease())
                    .png("b32")
                    .height(height)
                    .ascent(ascent)
                    .width(1)
                    .build())
            .offset64(
                ConfiguredChar.builder()
                    .character(CharacterArranger.getAndIncrease())
                    .png("b64")
                    .height(height)
                    .ascent(ascent)
                    .width(1)
                    .build())
            .offset128(
                ConfiguredChar.builder()
                    .character(CharacterArranger.getAndIncrease())
                    .png("b128")
                    .height(height)
                    .ascent(ascent)
                    .width(1)
                    .build())
            .right(
                ConfiguredChar.builder()
                    .character(CharacterArranger.getAndIncrease())
                    .png("b0")
                    .height(14)
                    .ascent(7)
                    .width(1)
                    .build())
            .build();

    if (!registerBackGround(key, background)) {
      warn("Found duplicated background: " + key);
    }
  }

  public boolean registerBackGround(@NotNull String key, @NotNull Background backGround) {
    if (backgroundMap.containsKey(key)) {
      return false;
    }
    backgroundMap.put(key, backGround);
    return true;
  }

  private void saveDefaultBackgrounds() {
    String[] bgList = new String[] {"b0", "b1", "b2", "b4", "b8", "b16", "b32", "b64", "b128"};
    for (String bg : bgList) {
      SR.getPlugin().saveResource("backgrounds" + File.separator + bg + ".png", false);
    }
  }
}
