package me.unurled.sacredrealms.sr.config;

import static me.unurled.sacredrealms.sr.utils.Logger.error;

import java.io.File;
import java.util.List;
import me.unurled.sacredrealms.sr.SR;
import me.unurled.srcore.api.Manager;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.minimessage.MiniMessage;
import org.bukkit.configuration.ConfigurationSection;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public enum Settings {
  REDIS_HOST("redis.host"),
  REDIS_PORT("redis.port"),
  RESOURCE_PACK_URL("resource_pack"),
  RESOURCE_PACK_API("resource_pack_api");

  private final String path;

  Settings(String path) {
    this.path = path;
  }

  public String getPath() {
    return path;
  }

  public @Nullable Object getValue() {
    ConfigManager cm = Manager.getInstance(ConfigManager.class);
    if (cm != null) {
      return cm.getConfig().get(path);
    }
    return null;
  }

  public void setValue(Object value) {
    SR.getPlugin().getConfig().set(path, value);
    try {
      SR.getPlugin().getConfig().save(new File(SR.getPlugin().getDataFolder(), "config.yml"));
    } catch (Exception e) {
      error("Failed to save config.yml");
    }
  }

  @Override
  public String toString() {
    try {
      return (String) getValue();

    } catch (NullPointerException e) {
      return "";
    }
  }

  @Nullable
  public Integer toInt() {
    try {
      return (int) getValue();
    } catch (NullPointerException e) {
      return null;
    }
  }

  public @NotNull Component toComponent() {
    Object value = getValue();
    if (value != null) {
      return MiniMessage.miniMessage().deserialize(value.toString());
    } else {
      return Component.empty();
    }
  }

  public Boolean toBool() {
    return (Boolean) getValue();
  }

  public List<String> toStringList() {
    ConfigManager cm = Manager.getInstance(ConfigManager.class);
    if (cm != null) {
      return cm.getConfig().getStringList(path);
    }
    return List.of();
  }

  public @Nullable ConfigurationSection toConfigSection() {
    ConfigManager cm = Manager.getInstance(ConfigManager.class);
    if (cm != null) {
      return cm.getConfig().getConfigurationSection(path);
    }
    return null;
  }
}
