package me.unurled.sacredrealms.sr.gui.difficulty;

import me.unurled.sacredrealms.sr.components.difficulty.Difficulty;
import me.unurled.sacredrealms.sr.components.player.PlayerManager;
import me.unurled.sacredrealms.sr.components.player.SRPlayer;
import me.unurled.srcore.api.Manager;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.event.inventory.ClickType;
import org.bukkit.event.inventory.InventoryClickEvent;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.NotNull;
import xyz.xenondevs.invui.item.Item;
import xyz.xenondevs.invui.item.ItemProvider;
import xyz.xenondevs.invui.item.builder.ItemBuilder;
import xyz.xenondevs.invui.item.impl.AbstractItem;
import xyz.xenondevs.invui.window.Window;

public class DifficultyItem extends AbstractItem {

  private final Difficulty difficulty;

  public DifficultyItem(Difficulty difficulty) {
    this.difficulty = difficulty;
  }

  /**
   * Gets the {@link ItemProvider}. This method gets called every time a {@link Window} is notified
   * ({@link #notifyWindows()}).
   *
   * @return The {@link ItemProvider}
   */
  @Override
  public ItemProvider getItemProvider() {
    return switch (difficulty) {
      case EASY ->
          new ItemBuilder(Material.BREAD)
              .setDisplayName("Easy")
              .addLoreLines(
                  "Is you are a new player or",
                  "someone who just want to",
                  "enjoy the game without " + "grinding too much",
                  "",
                  "Click Here!");
      case NORMAL ->
          new ItemBuilder(Material.GRASS_BLOCK)
              .setDisplayName("Normal")
              .addLoreLines(
                  "Is you are a regular player",
                  "who want to enjoy the game",
                  "without too much grinding",
                  "",
                  "Click Here!");
      case HARD ->
          new ItemBuilder(Material.DIAMOND_SWORD)
              .setDisplayName("Hard")
              .addLoreLines(
                  "Is you are a hardcore player",
                  "who want to enjoy the game",
                  "with some grinding",
                  "",
                  "Click Here!");
      case INSANE ->
          new ItemBuilder(Material.NETHERITE_SWORD)
              .setDisplayName("Insane")
              .addLoreLines(
                  "Is you are a pro player",
                  "who want to enjoy the game",
                  "with a lot of grinding",
                  "some things are hidden very well here...",
                  "",
                  "Click Here!");
    };
  }

  /**
   * A method called if the {@link ItemStack} associated to this {@link Item} has been clicked by a
   * player.
   *
   * @param clickType The {@link ClickType} the {@link Player} performed.
   * @param player The {@link Player} who clicked on the {@link ItemStack}.
   * @param event The {@link InventoryClickEvent} associated with this click.
   */
  @Override
  public void handleClick(
      @NotNull ClickType clickType, @NotNull Player player, @NotNull InventoryClickEvent event) {
    // Set the player's difficulty
    PlayerManager pm = Manager.getInstance(PlayerManager.class);
    if (pm.isSRPlayer(player.getUniqueId())) {
      SRPlayer sr = pm.getPlayer(player.getUniqueId());
      if (sr != null) {
        sr.setDifficulty(difficulty);
        player.closeInventory();
        player.sendMessage("Set difficulty to " + difficulty.name() + "!");
      }
    }
  }
}
