package me.unurled.sacredrealms.sr.gui.treasure;

import static me.unurled.sacredrealms.sr.utils.Component.textComp;
import static me.unurled.sacredrealms.sr.utils.Logger.log;

import java.util.HashMap;
import java.util.Map;
import me.unurled.sacredrealms.sr.components.treasure.Treasure;
import me.unurled.sacredrealms.sr.components.treasure.TreasureManager;
import me.unurled.srcore.api.Manager;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.event.inventory.ClickType;
import org.bukkit.event.inventory.InventoryClickEvent;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.jetbrains.annotations.NotNull;
import xyz.xenondevs.invui.item.Item;
import xyz.xenondevs.invui.item.ItemProvider;
import xyz.xenondevs.invui.item.builder.ItemBuilder;
import xyz.xenondevs.invui.item.impl.AbstractItem;
import xyz.xenondevs.invui.window.Window;

public class ConfirmItem extends AbstractItem {

  private final Treasure treasure;

  public ConfirmItem(Treasure treasure) {
    this.treasure = treasure;
  }

  /**
   * Gets the {@link ItemProvider}. This method gets called every time a {@link Window} is notified
   * ({@link #notifyWindows()}).
   *
   * @return The {@link ItemProvider}
   */
  @Override
  public ItemProvider getItemProvider() {
    ItemStack confirm = new ItemStack(Material.GREEN_STAINED_GLASS_PANE);
    ItemMeta confirmMeta = confirm.getItemMeta();
    confirmMeta.displayName(textComp("<b><green>Confirm</green></b>"));
    confirm.setItemMeta(confirmMeta);
    return new ItemBuilder(confirm);
  }

  /**
   * A method called if the {@link ItemStack} associated to this {@link Item} has been clicked by a
   * player.
   *
   * @param clickType The {@link ClickType} the {@link Player} performed.
   * @param player The {@link Player} who clicked on the {@link ItemStack}.
   * @param event The {@link InventoryClickEvent} associated with this click.
   */
  @Override
  public void handleClick(
      @NotNull ClickType clickType, @NotNull Player player, @NotNull InventoryClickEvent event) {
    Map<Integer, ItemStack> items = new HashMap<>();
    for (int i = 0; i < player.getInventory().getContents().length; i++) {
      if (i < 9 || i > 35) {
        log("Skipping slot " + i);
        continue;
      }
      if (event.getClickedInventory() == null) {
        log("Clicked inventory is null");
        continue;
      }
      ItemStack content = event.getClickedInventory().getItem(i);
      if (content != null && content.getType() != Material.AIR) {
        items.put(i, content);
        log("Added item to items: " + content + " at slot " + i);
      }
    }
    log("Items: " + items);
    TreasureManager tm = Manager.getInstance(TreasureManager.class);
    tm.setBlocks(treasure.getId(), items);

    player.closeInventory();
  }
}
