package me.unurled.sacredrealms.sr.utils;

import java.io.FileInputStream;
import java.io.IOException;
import java.security.DigestInputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;
import org.jetbrains.annotations.NotNull;

public class File {

  private File() {}

  public static @NotNull String getFileChecksum(String filePath)
      throws IOException, NoSuchAlgorithmException {
    MessageDigest sha = MessageDigest.getInstance("MD5");

    try (FileInputStream fis = new FileInputStream(filePath)) {
      DigestInputStream dis = new DigestInputStream(fis, sha);
      while (dis.read() != -1)
        ;
      dis.close();
    }

    // Convert the byte array to a hexadecimal string
    byte[] bytes = sha.digest();
    StringBuilder sb = new StringBuilder();
    for (byte aByte : bytes) {
      sb.append(String.format("%02x", aByte));
    }

    return sb.toString();
  }

  public static void zipFile(
      @NotNull java.io.File fileToZip, String fileName, ZipOutputStream zipOut) throws IOException {
    if (fileToZip.isHidden()) {
      return;
    }
    try (FileInputStream fis = new FileInputStream(fileToZip)) {
      ZipEntry zipEntry = new ZipEntry(fileName);
      zipEntry.setTime(0);
      zipOut.putNextEntry(zipEntry);
      byte[] bytes = new byte[1024];
      int length;
      while ((length = fis.read(bytes)) >= 0) {
        zipOut.write(bytes, 0, length);
      }
      zipOut.closeEntry();
    }
  }

  public static void zipDirectory(
      @NotNull java.io.File folderToZip, String folderName, ZipOutputStream zipOut)
      throws IOException {
    java.io.File[] children = folderToZip.listFiles();
    if (children == null) {
      return;
    }
    for (java.io.File childFile : children) {
      if (childFile.isDirectory()) {
        zipDirectory(childFile, folderName + "/" + childFile.getName(), zipOut);
      } else {
        zipFile(childFile, folderName + "/" + childFile.getName(), zipOut);
      }
    }
  }
}
